<?php

namespace Drupal\drush_tools\Commands;

use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Drush\Commands\DrushCommands;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://cgit.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://cgit.drupalcode.org/devel/tree/drush.services.yml
 */
class DrushToolsCommands extends DrushCommands {

  /**
   * Remove Duplicate URL aliases
   *
   * @param $content_type
   *   Specify a content type for which the duplicate aliases should be removed.
   * @param array $options
   *   An associative array of options whose values come from cli, aliases, config, etc.
   * @option option-name
   *   Description
   * @usage drush_tools-remove-duplicate-aliases foo
   *   Usage description
   *
   * @command drush_tools:remove-duplicate-aliases
   * @aliases rda
   */
  public function commandName($content_type = "none", $options = ['option-name' => 'default']) {
    $query = \Drupal::entityQuery('node');
    if ("none" != $content_type) {
      $query->condition('type', $content_type, '=');
    }
    $nids = $query->execute();
    $path_alias_storage = \Drupal::entityTypeManager()->getStorage('path_alias');

    foreach ($nids as $nid) {
      $actual_alias = \Drupal::service('path.alias_manager')
        ->getAliasByPath('/node/' . $nid);

      // Load all path alias for this node.
      $alias_objects = $path_alias_storage->loadByProperties([
        'path' => '/node/' . $nid,
      ]);

      // Delete all other alias than the actual one.
      foreach ($alias_objects as $alias_object) {
        if ($alias_object->get('alias')->value !== $actual_alias) {
          $alias_object->delete();
        }
      }
      // Load all new path alias for this node.
      $new_alias_objects = $path_alias_storage->loadByProperties([
        'path' => '/node/' . $nid,
      ]);
      // Delete duplicate aliases.
      if (count($new_alias_objects) > 1) {
        array_shift($new_alias_objects);
        foreach ($new_alias_objects as $alias_object) {
          $alias_object->delete();
        }
      }
    }
    $this->logger()->success(dt('Duplicate Aliases removed.'));
  }

  /**
   * An example of the table output format.
   *
   * @param array $options An associative array of options whose values come from cli, aliases, config, etc.
   *
   * @field-labels
   *   group: Group
   *   token: Token
   *   name: Name
   * @default-fields group,token,name
   *
   * @command drush_tools:token
   * @aliases token
   *
   * @filter-default-field name
   * @return \Consolidation\OutputFormatters\StructuredData\RowsOfFields
   */
  public function token($options = ['format' => 'table']) {
    $all = \Drupal::token()->getInfo();
    foreach ($all['tokens'] as $group => $tokens) {
      foreach ($tokens as $key => $token) {
        $rows[] = [
          'group' => $group,
          'token' => $key,
          'name' => $token['name'],
        ];
      }
    }
    return new RowsOfFields($rows);
  }
}
